#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a new image framed (using G'MIC artistic filters)

# (c) Dinasset 2023
#
#   History:
#
#   v 1.1.0: 2023-08: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
import random
from random import randint
from gimpfu import *

# define here some constant value, when required
maxwh=5000

############################################################################
# Service routines
#---------------------------------------------------------------------------
# from Original drawable create the Output image
#---------------------------------------------------------------------------
def createOutimage(inImage, inLayer, widthheight, prepstep):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    
   # define/calculate the scale factor for the (new) output image -------------------------
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have output size same as input size
        if realWidthHeight<=maxwh:
            widthheight = realWidthHeight
        else:
            widthheight = maxwh  # max
            gimp.message ("forced max width+height")
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = (optiWidthHeight*1.0)/(realWidthHeight*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size ------------------------------------------------
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
    
    # create base for effect
    pdb.gimp_image_set_active_layer (newImage,  newBG)

    # optional user choice for preprocessing
    if prepstep>0:  #user did a selection
        #               #"DCP Dehaze (good for light image",
        #               #"Dynamic Range Increase",
        #               #"Equalize Local Histogram",
        #               #"Local Contrast Enhancement",
        #               #"Local Normalization",
        #               #"Local Processing (Equalize)",
        #               #"Local Processing (Normalize)",
        if prepstep==1: # jeje_dehaze 5,1,0.2,1,0,0,0,0,0
            parms='5,1,0.2,1,0,0,0,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -jeje_dehaze "+str(parms)+" ")
        elif prepstep==2: # fx_tk_dri 0.5,0.5,0.5,2.5,0.5,1,1,0
            parms='0.5,0.5,0.5,2.5,0.5,1,1,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_tk_dri "+str(parms)+" ")
        elif prepstep==3: # fx_equalize_local_histograms 75,2,4,100,8,1,16,0,50,50
            parms='75,2,4,100,8,1,16,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_equalize_local_histograms "+str(parms)+" ")
        elif prepstep==4: # fx_LCE 80,0.5,1,1,0,0
            parms='80,0.5,1,1,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_LCE "+str(parms)+" ")
        elif prepstep==5: # fx_normalize_local 2,6,5,20,1,11,0,50,50
            parms='2,6,5,20,1,11,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_normalize_local "+str(parms)+" ")
        elif prepstep==6: # fx_local_processing 1,75,10,50,20,0,7,0,50,50
            parms='1,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
        elif prepstep==7: # fx_local_processing 0,75,10,50,20,0,7,0,50,50
            parms='0,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
    newBG = pdb.gimp_image_get_active_layer (newImage)
            
    # adjust mean value of the base
    # "normal" mean value should be betweeb 117.5 and 137.5
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_LUMINANCE, 0.0, 1.0)
    #gimp.message ("mean="+str(mean))
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean<117.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.1, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean>137.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.9, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    newBG.opacity = 100.0
    
    return(newImage)
#--------------------------------------------------------------------------- 
    
############################################################################
# Service routine (by MareroQ) for Frame colour
#---------------------------------------------------------------------------
def gmic_rgba_to_rgb(rgba_color):

	r = rgba_color[0]
	g = rgba_color[1]
	b = rgba_color[2]

	# write rgb in correct syntax
	rgb_color = "{0},{1},{2}".format(r,g,b)
	return rgb_color
	     

############################################################################
# Main
#---------------------------------------------------------------------------

def ArtisticFramed (inImage, inLayer,
                    widthheight,
                    frame, size_fr, color_frame,
                    style, prepstep,
                    doFlatten):
    
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    # auto mode and opacity
    artlayermode=0	            #fixed auto
    artlayeropacity=0		    #fixed auto
    	
    newImage = createOutimage (inImage, inLayer, widthheight, prepstep)
    newBG = newImage.layers[0]
    newBG.name = "new BG Prepared"
    Original = newImage.layers[1]
    imgW = newImage.width
    imgH = newImage.height
    imgsize = imgW+imgH
     
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply fixed GMIC Segmentation
    # fx_segment_watershed 4.95,1,0,0,0,50,50
    parms='4.95,1,0,0,0,50,50 '
    pdb.plug_in_gmic_qt(newImage, newBG, 1,2,
    #   "-v - -fx_segment_watershed 4.95,1,0,0,0,50,50 ")
        "-v - -fx_segment_watershed "+str(parms)+" ")
    baseforeffect = pdb.gimp_image_get_active_layer(newImage)
    baseforeffect.name = "new BG Prepared, Painted"
        
    #pdb.gimp_display_new (newImage)
    #return
        
    # apply chosen GMIC Artistic effect
    autolayermode = LAYER_MODE_GRAIN_MERGE  # default mode
    autolayeropacity = 75.0  # default opacity
    
    # COMMON START
    StyleLayer = baseforeffect.copy()
    newImage.add_layer (StyleLayer, 0)
    
    if style==0:    #Angoisse
        # apply Angoisse 
        #[G'MIC] samj_Angoisse 1,5,0,5,100,2,4,1,250
        parms='1,5,0,5,100,2,4,1,250'     
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_Angoisse "+str(parms)+" ")
        StyleLayer.name = "Angoisse Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==1:    #Aurora
        # apply Aurora 
        #[G'MIC] gcd_aurora 6,1,0
        parms='6,1,0'     
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -gcd_aurora "+str(parms)+" ")
        StyleLayer.name = "Aurora Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LINEAR_LIGHT
        if artlayeropacity<1: autolayeropacity = 85.0
        
    elif style==2:    #Barbouillage
        # apply BARBOUILLAGE
        #[G'MIC] samj_Barbouillage_Paint_Daub 2,2,100,0.2,1,4,1,0,8
        parms='2,2,100,0.2,1,4,1,0,8'
        
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_Barbouillage_Paint_Daub "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)         
        StyleLayer.name = "Barbouillage Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 95.0
   
    elif style==3:    #Crayongraffiti
        # apply BLACK CRAYON GRAFFITI
        #[G'MIC] fx_crayongraffiti2 40,100,0,0.5,0,0,1,2,0
        parms='40,100,0,0.5,0,0,1,2,0'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_crayongraffiti2 "+str(parms)+" ")
        StyleLayer.name = "Graffiti Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if artlayeropacity<1: autolayeropacity = 50.0
              
    elif style==4:    #Blockism
        # apply BLOCKISM
        #[G'MIC] fx_blockism 3,1.6,0.5,50,0.5,64,0,0,0
        parms='3,1.6,0.5,50,0.5,64,0,0,0'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_blockism "+str(parms)+" ")
        StyleLayer.name = "Blockism Layer "+str(parms)
              
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_SATURATION
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==5:    #Bokeh
        # apply Bokeh 
        #[G'MIC] fx_bokeh 3,8,0,30,8,4,0.3,0.2,210,210,80,160,0.7,30,20,20,1,2,170,130,20,110,0.15,0,50,50
        parms='3,8,0,30,8,4,0.3,0.2,210,210,80,160,0.7,30,20,20,1,2,170,130,20,110,0.15,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_bokeh "+str(parms)+" ")
        StyleLayer.name = "Bokeh Layer "+str(parms)
         
        if artlayermode==0: autolayermode = LAYER_MODE_HARDLIGHT
        if artlayeropacity<1: autolayeropacity = 95.0
         
    elif style==6:    #Brushify
        # apply BRUSHIFY
        #[G'MIC] fx_brushify 7,0.25,4,64,25,12,0,2,4,0.2,0.5,30,1,1,1,5,0,0.2,1
        parms='7,0.25,4,64,25,12,0,2,4,0.2,0.5,30,1,1,1,5,0,0.2,1'
                  
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)         
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_brushify "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)                 
        StyleLayer.name = "Brushify Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 95.0
              
    elif style==7:    #Cartoon
        # apply CARTOON
        #[G'MIC] cartoon 3,200,10,0.25,1.5,8,0,50,50
        parms='3,200,10,0.25,1.5,8,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - cartoon "+str(parms)+" ")
        StyleLayer.name = "Cartoon Layer "+str(parms) 
         
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 95.0
              
    elif style==8:    #ChalkItUp
        # apply CHALKITUP
        #[G'MIC] samj_chalkitup 10,2.5,5,50,1,5,5,0,0,7,0.8,1.9,7,0
        parms='10,2.5,5,50,1,5,5,0,0,7,0.8,1.9,7,0'
              
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_chalkitup "+str(parms)+" ")
        StyleLayer.name = "Chalkitup Layer "+str(parms)
              
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if artlayeropacity<1: autolayeropacity = 35.0
              
    elif style==9:    #CircleAbstraction
        # apply CIRCLE ABSTRACTION
        #[G'MIC] fx_circle_abstraction 16,4,0.8,0,1,1,1,0,50,50
        parms='16,4,0.8,0,1,1,1,0,50,50'
              
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)         
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_circle_abstraction "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)         
        StyleLayer.name = "CircleAbstr Layer "+str(parms)
         
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==10:    #ColorAbstraction
        # apply COLOR ABSTRACTION PAINT
        #[G'MIC] fx_ColorAbstractionPaint 5,10,1,0,1,0,1,0,0,0,0,0,0,0
        parms='5,10,1,0,1,0,1,0,0,0,0,0,0,0'
              
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)         
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ColorAbstractionPaint "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)         
        StyleLayer.name = "ColorAbstr Layer "+str(parms)
              
        if artlayermode==0: autolayermode = LAYER_MODE_HSL_COLOR
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==11:    #Colored Pencils
        # apply COLORED PENCILS
        #[G'MIC] fx_cpencil 3,4,255,0,2,1,0
        parms='3,4,255,0,2,1,0'
              
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_cpencil "+str(parms)+" ")
        StyleLayer.name = "ColorPencil Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if artlayeropacity<1: autolayeropacity = 80.0
        
    elif style==12:    #Comic Book
        # apply COMICBOOK
        #[G'MIC] cl_comic 0,2,0,1,1,15,15,1,10,20,6,2,0,0,0,0,0,0,50,50
        parms='0,2,0,1,1,15,15,1,10,20,6,2,0,0,0,0,0,0,50,50'
              
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)         
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -cl_comic "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)           
        StyleLayer.name = "Comic Layer "+str(parms)

        if artlayermode==0: autolayermode = LAYER_MODE_HARD_MIX
        if artlayeropacity<1: autolayeropacity = 65.0
        
    elif style==13:    #Cubism
        # apply CUBISM
        #[G'MIC] fx_cubism 2,50,10,90,0.7,0,0,50,50
        parms='2,50,10,90,0.7,0,0,50,50'
              
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_cubism "+str(parms)+" ")
        StyleLayer.name = "Cubism Layer "+str(parms)
              
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==14:    #Cutout
        # apply CUTOUT
        #[G'MIC] fx_cutout 4,0.5,4,1,0,50,50 
        parms='4,0.5,4,1,0,50,50'
              
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_cutout "+str(parms)+" ")
        StyleLayer.name = "Cutout Layer "+str(parms)
              
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 50.0
                                           
    elif style==15:    #Diffusion Tensors 
        # apply DIFFUSION TENSORS
        #[G'MIC] fx_diffusiontensors 10,5,2,1,0,1,0,3,0,50,50
        parms='10,5,2,1,0,1,0,3,0,50,50'
              
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_diffusiontensors "+str(parms)+" ")
        StyleLayer.name = "DTensors Layer "+str(parms)
              
        if artlayermode==0: autolayermode = LAYER_MODE_OVERLAY
        if artlayeropacity<1: autolayeropacity = 75.0
                                            
    elif style==16:    #Doodle
        # apply DOODLE
        #[G'MIC] fx_doodle 30,2,2,1.5,0,70,0
        parms='30,2,2,1.5,0,70,0'    

        #[G'MIC] # increase details
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_tk_dri 0,0,0,5,1,1,1,0 ")
        # reduce size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,       
                              StyleLayer.height/2,      
                              True)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_doodle "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "Doodle Layer "+str(parms)

        if artlayermode==0: autolayermode = LAYER_MODE_GRAIN_MERGE  
        if artlayeropacity<1: autolayeropacity = 95.0    
            
    #pdb.gimp_display_new (newImage)
    #return
                                           
    elif style==17:    #DreamSmoothing
        # apply DREAM SMOOTHING
        #[G'MIC] fx_dreamsmooth 3,1,1,0.8,0,0.8,1,24,0
        parms='3,1,1,0.8,0,0.8,1,24,0'    

        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/3*2,    
                              StyleLayer.height/3*2,
                              True)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_dreamsmooth "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "DreamSmooth Layer "+str(parms)

        if artlayermode==0: autolayermode = LAYER_MODE_SCREEN    
        if artlayeropacity<1: autolayeropacity = 75.0
                                           
    elif style==18:    #Ellipsionism
        # apply ELLIPSIONISM
        #[G'MIC] fx_ellipsionism 20,10,0.5,1,3,0.5,0,50,50
        parms='20,10,0.5,1,3,0.5,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ellipsionism "+str(parms)+" ")
        #[G'MIC] # increase details
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_tk_dri 0,0,0,5,1,1,1,0 ")
        StyleLayer.name = "Ellipsionism Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE 
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==19:    #Felt Pen
        # apply FELT PEN
        #[G'MIC] fx_feltpen 400,100,4,0.4,20,5,0,50,50
        parms='400,100,4,0.4,20,5,0,50,50'
        
        #[G'MIC] # v.2 increase details
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_tk_dri 0,0,0,5,1,1,1,0 ")    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_feltpen "+str(parms)+" ")
        
        StyleLayer.name = "FeltPen Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LINEAR_BURN  #LUMINANCE
        if artlayeropacity<1: autolayeropacity = 65.0   # was 50.0
                                           
    elif style==20:    #FingerPaint
        # apply FINGER PAINT
        #[G'MIC] gtutor_fpaint 0.35,0.45,0,0,45,0.5,0.5,0.5,0
        parms='0.35,0.45,0,0,45,0.5,0.5,0.5,0'
        
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -gtutor_fpaint "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "FingerPaint Layer "+str(parms) 
        
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if artlayeropacity<1: autolayeropacity = 95.0
         
    elif style==21:    #Fractalize
        # apply FRACTALIZE
        #[G'MIC] fractalize 0.1

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fractalize 0.1 ")
    
        StyleLayer.name = "Fractal Layer 0.1"
    
        if artlayermode==0: autolayermode = LAYER_MODE_HARDLIGHT
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==22:    #Ghost
        # apply GHOST
        #[G'MIC] fx_ghost 200,2,2,1,3,16,1,0,50,50
        parms='200,2,2,1,3,16,1,0,50,50'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ghost "+str(parms)+" ")
    
        StyleLayer.name = "Ghost Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_MULTIPLY
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==23:    #Graphic Boost
        # apply GRAPHIC BOOST
        #[G'MIC] fx_graphic_boost4 1.25,10,0,0.15,14,0,1,0.5,0.45,2,0,0,0,1,1,1,0.5,0.45,1
        parms='1.25,10,0,0.15,14,0,1,0.5,0.45,2,0,0,0,1,1,1,0.5,0.45,1'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_graphic_boost4 "+str(parms)+" ")
    
        StyleLayer.name = "GBoost Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LINEAR_BURN
        if artlayeropacity<1: autolayeropacity = 55.0
        
    elif style==24:    #Graphic Novel
        # apply GRAPHIC NOVEL
        #[G'MIC] fx_graphic_novelfxl 0,2,6,5,20,0,0.62,14,0,1,0.5,0.78,1.92,0,0,0,1,1,1,0.5,0.8,1.28
        parms='0,2,6,5,20,0,0.62,14,0,1,0.5,0.78,1.92,0,0,0,1,1,1,0.5,0.8,1.28'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_graphic_novelfxl "+str(parms)+" ")
        pdb.gimp_drawable_brightness_contrast (StyleLayer, -0.1, +0.2)
   
        StyleLayer.name = "GNovel Layer "+str(parms)
   
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if artlayeropacity<1: autolayeropacity = 95.0
                                                                                                         
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==25:    #Hard Sketch
        # apply HARD SKETCH
        #[G'MIC] fx_hardsketchbw 300,100,1,0.1,1,0,4,0,50,50
        parms='300,100,1,0.1,1,0,4,0,50,50'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_hardsketchbw "+str(parms)+" ")
        pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE,
                                     StyleLayer, (1.0,1.0,1.0,1.0))
        pdb.gimp_drawable_edit_clear (StyleLayer)
        pdb.gimp_selection_none(newImage)
        pdb.gimp_drawable_brightness_contrast (StyleLayer, -0.4, +0.2)
    
        StyleLayer.name = "HSketch Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 75.0
                                                                                                         
    #pdb.gimp_display_new (newImage)
    #return
    
    elif style==26:    #Highlight Bloom
        # apply Highlight Bloom
        #[G'MIC] fx_highlight_bloom 400,0,0,100,100,0,50,50
        parms='400,0,0,100,100,0,50,50'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_highlight_bloom "+str(parms)+" ")
        StyleLayer.name = "HBloom Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_HARDLIGHT
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==27:    #Hope Poster
        # apply HOPE POSTER
        #[G'MIC] fx_poster_hope 0,20,0,50,50
        parms='0,20,0,50,50'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_poster_hope "+str(parms)+" ")
    
        StyleLayer.name = "HPoster Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LCH_HUE
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==28:    #HoughSketch
        # apply HoughSketch         
        #[G'MIC] fx_houghsketchbw 1.25,35,7,75,0.1,4,0,50,50
        # it seems that this GMIC function did not work properly
        # on a layer with straight lines (like when framed)
        # *******************************************************
        # *******************************************************
        parms='1.25,35,7,75,0.1,4,0,50,50'
        
        # use modified David's filter (29th of May 23)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_houghsketchbw "+str(parms)+" ")
        StyleLayer.name = "HSketch Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LUMA_DARKEN_ONLY
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==29:    #Illustration Look
        # Apply IllustrationLook
        #[G'MIC] fx_illustration_look 100,0,0,0,0
        parms='100,0,0,0,0'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_illustration_look "+str(parms)+" ")
        
        StyleLayer.name = "IllustrationLook Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==30:    #Kuwahara
        # Apply Kuwahara
        #[G'MIC] fx_kuwahara 2,5,0,2,0,50,50
        parms='2,5,0,2,0,50,50'
   
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_kuwahara "+str(parms)+" ")
        
        StyleLayer.name = "Kuwahara  "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LUMA_LIGHTEN_ONLY
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==31:    #Line Art
        # apply Photo to Line Art
        #[G'MIC] cl_lineart 4,1,2,1,19,15,0,0,6,2,2,0,0,0,50,50
        parms='4,1,2,1,19,15,0,0,6,2,2,0,0,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -cl_lineart  "+str(parms)+" ")
    
        StyleLayer.name = "LineArt  "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if artlayeropacity<1: autolayeropacity = 95.0
                                           
    elif style==32:    #Linify
        # apply LINIFY
        #[G'MIC] fx_linify 40,2,40,10,24,0,0,0,50,50
        parms='40,2,40,10,24,0,0,0,50,50'

        # reduce size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_linify "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "Linify Layer "+str(parms)

        if artlayermode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if artlayeropacity<1: autolayeropacity = 95.0
       
    elif style==33:    #Lyle Painting
        # apply Lyle Painting
        #[G'MIC] fx_lylejk_painting 10,2,4,100,0,50,50
        parms='10,2,4,100,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_lylejk_painting "+str(parms)+" ")
    
        StyleLayer.name = "LylePaint Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 100.0
     
    elif style==34:    #Squiggly
        # apply Make Squiggly
        #[G'MIC] fx_Squiggly 20,12,0,0,1.5,1,0,0,3,0.2,0.4,0,1,1,0
        parms='20,12,0,0,1.5,1,0,0,3,0.2,0.4,0,1,1,0'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_Squiggly "+str(parms)+" ")
    
        StyleLayer.name = "Squiggly Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if artlayeropacity<1: autolayeropacity = 75.0
         
    elif style==35:    #Morphology Paint
        # apply Morphology Paint
        #[G'MIC] fx_MorphoPaint 1,18,2,25,100,230,8,3,4,0.5,2,0.5,200,1,1,1,1,1,1,1,1,0
        parms='1,18,2,25,100,230,8,3,4,0.5,2,0.5,200,1,1,1,1,1,1,1,1,0'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_MorphoPaint "+str(parms)+" ")
    
        StyleLayer.name = "MorphoPaint Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 100.0
        
    elif style==36:    #Paint with Brush
        # apply Paint with Brush (smooth)
        #[G'MIC] fx_paint_with_brush 0,"0",1,16,0,100,100,10,80,0.5,3,45,0,6,2,10,0,0,0,60,20,1,0,0,0,30,15,15,15,15,15,15,1,45,0,0,50,50,1,0
        parms1='7,"0",1,16,0,100,100,10,80,0.5,3,45,0,6,2,10,0,0,0,60,'
        parms2='20,1,0,0,0,30,15,15,15,15,15,15,1,45,0,0,50,50,1,0'
        parms=str(parms1)+str(parms2)
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_paint_with_brush "+str(parms)+" ")
    
        StyleLayer.name = "PaintwBrush Layer predefined smooth (dft parms)"
    
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 100.0
        
    elif style==37:    #Painting 
        # apply Painting
        #[G'MIC] fx_painting 5,2.5,1.5,50,1,0,50,50
        parms='5,2.5,1.5,50,1,0,50,50'  

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_painting "+str(parms)+" ")
    
        StyleLayer.name = "Painting Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if artlayeropacity<1: autolayeropacity = 35.0
        
    elif style==38:    #Pastel Art 
        # apply Pastel Art
        #[G'MIC] fx_pastell 0.6,1,0,20,30,300,1,30,1,1,0.5,0,0,10,3,0,9,3,0,12,0
        parms='0.6,1,0,20,30,300,1,30,1,1,0.5,0,0,10,3,0,9,3,0,12,0'  
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_pastell "+str(parms)+" ")
    
        StyleLayer.name = "PastelArt Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 100.0
        
    elif style==39:    #Pen Drawing 
        # apply Pen Drawing
        #[G'MIC] fx_pen_drawing 18,0,50,50
        parms='18,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_pen_drawing "+str(parms)+" ")
    
        StyleLayer.name = "PenDraw Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 100.0
         
    elif style==40:    #Photo Illustration 
        # apply Photo Illustration
        #[G'MIC] fx_tk_photoillustration 0,0.5,0,0.5,0.5,0.5,1,0,5,0,1,0,0,1,0,0,1.2,98.5,5,0.5,0,0,0,0,0,1,0
        parms='0,0.5,0,0.5,0.5,0.5,1,0,5,0,1,0,0,1,0,0,1.2,98.5,5,0.5,0,0,0,0,0,1,0'
        StyleLayer.name = "PhotoIll Layer "+str(parms)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_tk_photoillustration "+str(parms)+" ")
        # note: it seems that after the application the layer name changes
        # hence, attributes have been moved before application
        
        if artlayermode==0: autolayermode = LAYER_MODE_VIVID_LIGHT
        if artlayeropacity<1: autolayeropacity = 70.0
        
    elif style==41:    #Poligonize Delauney 
        # apply Poligonize Delauney 
        #[G'MIC] fx_polygonize_delaunay 40,5,75,0.5,4,100,0,0,0,255,1,0,50,50
        parms='40,5,75,0.5,4,100,0,0,0,255,1,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_polygonize_delaunay "+str(parms)+" ")
        StyleLayer.name = "PolyDelauney Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 100.0
        
    elif style==42:    #Poligonize [Energy] 
        # apply Poligonize [Energy] 
        #[G'MIC] fx_polygonize 300,10,10,10,10,255,255,0,255,0,50,50
        parms='300,10,10,10,10,255,255,0,255,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_polygonize "+str(parms)+" ")
        StyleLayer.name = "Polygonize Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_OVERLAY
        if artlayeropacity<1: autolayeropacity = 100.0
        
    elif style==43:    #Poster Edges 
        # apply Poster Edges 
        #[G'MIC] fx_poster_edges 0,50,15,2,0,5,0,0,50,50
        parms='0,50,15,2,0,5,0,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_poster_edges "+str(parms)+" ")
        StyleLayer.name = "PosterEdges Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 100.0
        
    elif style==44:    #Posterize 
        # apply Posterize 
        #[G'MIC] fx_posterize 150,100,10,12,0,0,1,0,50,50
        parms='150,100,10,12,0,0,1,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_posterize "+str(parms)+" ")
        StyleLayer.name = "Posterize Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_HARD_MIX
        if artlayeropacity<1: autolayeropacity = 35.0
       
    elif style==45:    #PosterDither
        # apply Posterized Dithering
        #[G'MIC] fx_pdithered 1,1,0,0,20,1,5,1,0
        parms='1,1,0,0,20,1,5,1,0'
  
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_pdithered "+str(parms)+" ")
        StyleLayer.name = "PDither Layer "+str(parms)
  
        if artlayermode==0: autolayermode = LAYER_MODE_VIVID_LIGHT
        if artlayeropacity<1: autolayeropacity = 75.0
  
    elif style==46:    #Quadtree Variations
        # apply Quadtree Variations 
        #[G'MIC] fx_quadtree 2,4096,0.5,0,3,1.5,1,1,0,50,50
        parms='2,4096,0.5,0,3,1.5,1,1,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_quadtree "+str(parms)+" ")
        StyleLayer.name = "QuadtreeV Layer "+str(parms)

        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 70.0
        
    elif style==47:    #Rodilius
        # apply Rodilius 
        #[G'MIC] fx_rodilius 10,10,300,5,30,0,1,0,0,0,50,50
        parms='10,10,300,5,30,0,1,0,0,0,50,50'

        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_rodilius "+str(parms)+" ")
        StyleLayer.name = "Rodilius Layer "+str(parms)

        if artlayermode==0: autolayermode = LAYER_MODE_LIGHTEN_ONLY
        if artlayeropacity<1: autolayeropacity = 75.0
  
    elif style==48:    #Shapeism
        # apply Shapeism 
        #[G'MIC] fx_shapeism 2,7,0.38,0,1,5,32,8,3,1,5,0.5,1,0,0,0,255
        parms = '5,7,0.38,0,1,5,32,8,3,1,5,0.5,1,0,0,0,255'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - fx_shapeism "+str(parms)+" ")
        StyleLayer.name = "Shapeism Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_DODGE
        if artlayeropacity<1: autolayeropacity = 75.0
        
    elif style==49:    #Sharp Abstract
        # apply Sharp Abstract 
        #[G'MIC] fx_sharp_abstract 16,16,2,0,0,50,50
        parms='16,16,2,0,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_sharp_abstract "+str(parms)+" ")
        StyleLayer.name = "SharpAbstr Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_SUBTRACT
        if artlayeropacity<1: autolayeropacity = 75.0
  
    elif style==50:    #SimpleNoiseCanvas
        # apply SimpleNoiseCanvas 
        #[G'MIC] fx_SimpleNoiseCanvas 0,20,0,5,5,0,230,0,1,0,0,0,1,0,255,255,255,255,0
        parms = '0,20,0,5,5,0,230,0,1,0,0,0,1,0,255,255,255,255,0'
  
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_SimpleNoiseCanvas "+str(parms)+" ")
        StyleLayer.name = "SNoise Layer "+str(parms)
  
        if artlayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        if artlayeropacity<1: autolayeropacity = 95.0
  
    elif style==51:    #Skeletik
        # apply Skeletik 
        #[G'MIC] samj_Test_Skeletik 10,1,0,3,0.3,24,1,0,0,0,255
        parms = '10,1,0,3,0.3,24,1,0,0,0,255'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_Test_Skeletik "+str(parms)+" ")
        # it seems this function does not understand active layer
        StyleLayer = pdb.gimp_image_get_active_layer (newImage)
        StyleLayer.name = "Skeletik Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_DODGE
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==52:    #Sketch
        # apply Sketch 
        #[G'MIC] fx_sketchbw 3,45,180,30,1.75,0.02,0.5,0.75,0.15,0.7,3,6,1,1,4,5,0,50,50
        parms = '3,45,180,30,1.75,0.02,0.5,0.75,0.15,0.7,3,6,1,1,4,5,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_sketchbw "+str(parms)+" ")
        StyleLayer.name = "Sketch Layer "+str(parms)
    
        if artlayermode==0: autolayermode = LAYER_MODE_LINEAR_LIGHT
        if artlayeropacity<1: autolayeropacity = 95.0
  
    elif style==53:    #SmoothAbstract
        # apply SmoothAbstract 
        #[G'MIC] fx_smooth_abstract 20,2,20,1,50,0,50,50
        parms = '20,2,20,1,50,0,50,50'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - fx_smooth_abstract "+str(parms)+" ")
        StyleLayer.name = "Smooth Abstr Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_GRAIN_MERGE
        if artlayeropacity<1: autolayeropacity = 45.0
        
    elif style==54:  #Stylize
        # apply Stylize Portrait DeMetzinger (=example style 13)
        #[G'MIC] fx_stylize 13,5,0,0,0.5,2,3,0.5,0.1,3,3,0,0.7,1,0,1,0,5,5,7,1,30,2,2,1.85,0
        parms='13,5,0,0,0.5,2,3,0.5,0.1,3,3,0,0.7,1,0,1,0,5,5,7,1,30,2,2,1.85,0'
        
        stylezedname = [
                        "2 Braque  Landscape near Antwerp",
                        "3 Braque  Le Viaduc &agrave; L'Estaque",
                        "4 Braque  Little Bay at La Ciotat",
                        "5 Braque  The Mandola",
                        "6 Christine Garner  Black Colour Pencil",
                        "7 Christine Garner  Colour Pencil Sepia",
                        "8 Christine Garner  Dark Coloured Pencil",
                        "9 Christine Garner  Pencil",
                        "10 Christine Garner  Sketching Pastel",
                        "11 Christine Garner  Willow Charcoal",
                        "12 Delaunay  Windows Open Simultaneously",
                        "13 Delaunay  Portrait De Metzinger",
                        "14 Hokusai  The Great Wave",
                        "15 Kandinsky  Squares with Concentric Circles",
                        "16 Kandinsky  Yellow-Red-Blue",
                        "17 Klee  Death and Fire",
                        "18 Klee  In the Style of Kairouan",
                        "19 Klee  Oriental Pleasure Garden Anagoria",
                        "20 Klee  Polyphony 2",
                        "21 Klee  Red Waistcoat",
                        "22 Klimt  The Kiss",
                        "23 Mondrian  Composition in Red-Yellow-Blue",
                        "24 Mondrian  Evening; Red Tree",
                        "25 Mondrian  Gray Tree",
                        "26 Monet  San Giorgio Maggiore at Dusk",
                        "27 Monet  Water-Lily Pond",
                        "28 Monet  Wheatstacks - End of Summer",
                        "29 Munch  The Scream",
                        "30 Picabia  Udnie",
                        "31 Picasso  Les Demoiselles D'Avignon",
                        "32 Picasso  Seated Woman",
                        "33 Picasso  The Reservoir - Horta De Ebro",
                        "34 Pollock  Convergence",
                        "35 Pollock  Summertime Number 9A",
                        "36 Van Gogh  Almond Blossom",
                        "37 Van Gogh  Irises",
                        "38 Van Gogh  The Starry Night",
                        "39 Van Gogh  Wheat Field with Crows",    
                        ]
         
        parmsfull='13,5,0,0,0.5,2,3,0.5,0.1,3,3,0,0.7,1,0,1,0,5,5,7,1,30,2,2,1.85,0'
        # choose a random type (first parm)
        parms1=(random.randint(2,39))    # type of style (first 2 are "customer")
        parmsrest=',5,0,0,0.5,2,3,0.5,0.1,3,3,0,0.7,1,0,1,0,5,5,7,1,30,2,2,1.85,0'  #all the rest
        pdb.gimp_message ("stylize style chosen random="+str(stylezedname[parms1]))
        
        # reduced size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_stylize "+str(parms1)+str(parmsrest)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "Stylize Layer "+str(parms1)+str(parmsrest)
        
        if artlayermode==0: autolayermode = LAYER_MODE_LINEAR_LIGHT
        if artlayeropacity<1: autolayeropacity = 95.0
         
        #pdb.gimp_display_new (newImage)
        #return
         
        
    elif style==55:    #Vector Painting
        # apply Vector Painting 
        #[G'MIC] fx_vector_painting 9.7,0,50,50
        parms='9.7,0,50,50'
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_vector_painting "+str(parms)+" ")
        StyleLayer.name = "VectorPaint Layer "+str(parms)
        StyleLayer.opacity = autolayeropacity
        if artlayermode==0: autolayermode = LAYER_MODE_LUMINANCE
        if artlayeropacity<1: autolayeropacity = 95.0
        
    elif style==56:    #Wahrol
        # apply Warhol
        #[G'MIC] warhol 10,10,10,60
        parms='10,10,10,60'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -warhol "+str(parms)+" ")
        StyleLayer = pdb.gimp_image_get_active_layer (newImage) # lost ????
        StyleLayer.name = "Wahrol Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_HSL_COLOR
        if artlayeropacity<1: autolayeropacity = 25.0
        
    elif style==57:    #Whirl Drawing
        # apply Whirl Drawing 
        #[G'MIC] fx_draw_whirl 100,0,50,50
        parms='100,0,50,50'
        
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_draw_whirl "+str(parms)+" ")
        StyleLayer.name = "Whirl Drawing Layer "+str(parms)
        
        if artlayermode==0: autolayermode = LAYER_MODE_ADDITION
        if artlayeropacity<1: autolayeropacity = 95.0
                  
    #pdb.gimp_display_new (newImage)   
    #return
    #
    # =================================================================
    # final layer before outcome
    
    ArtisticLayer = pdb.gimp_image_get_active_layer(newImage)
    if artlayermode==0: ArtisticLayer.mode = autolayermode
    if artlayeropacity<1: ArtisticLayer.opacity = autolayeropacity
    final_layer = pdb.gimp_layer_new_from_visible (newImage, newImage, "Pre_Outcome")
    newImage.add_layer (final_layer, 0)
    final_layer.name = "Preliminary Outcome"
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (final_layer,
                                   HISTOGRAM_LUMINANCE, 0.0, 1.0)
    #pdb.gimp_message ("mean Artistic Layer="+str(mean))
    if mean < 75.5:
        final_layer.mode = LAYER_MODE_DODGE
    elif mean < 107.5:
        final_layer.mode = LAYER_MODE_ADDITION
    elif mean > 179.5:
        final_layer.mode = LAYER_MODE_MULTIPLY
    elif mean > 147.5:
        final_layer.mode = LAYER_MODE_OVERLAY
    else:    
        final_layer.mode = LAYER_MODE_NORMAL
        
    #pdb.gimp_display_new (newImage)
    #return
       
    # final step
     
    # outcome
    
    outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
    newImage.add_layer (outcome, 0)
    outcome.name = "Outcome (unframed)"
        
    #pdb.gimp_display_new (newImage)
    #return
       
    # apply GMIC Frame Blur  
    # fx_frame_blur 10,10,0,5,1,0,128,128,128,0,0,255,255,255,2,2,1,0,0.5,0.5,0
    hex = gmic_rgba_to_rgb(color_frame)
    SH = int(size_fr)
    SV = int(size_fr)
    Round=["0.0","1.0","0.2","0.15","0.1"]	
    Rnd=str(Round[frame])
    XSh = min(10,int(size_fr/6))
    YSh = min(10,int(size_fr/6))
    OuS = min(10,int(size_fr/3))	
    parms=str(SH)+','+str(SV)+',0,5,'+str(Rnd)+',0,128,128,128,0,'
    parms=parms+str(OuS)+','+str(hex)+','+str(XSh)+','+str(YSh)+',1,0,0.5,0.5,0 '	
    pdb.plug_in_gmic_qt(newImage, outcome, 1,2,
        "-v - -fx_frame_blur "+str(parms)+" ")
    Framed = pdb.gimp_image_get_active_layer(newImage)
    Framed.mode = LAYER_MODE_NORMAL
         
    if doFlatten==True:
        pdb.gimp_image_remove_layer (newImage, outcome)
        pdb.gimp_image_remove_layer (newImage, final_layer)
        pdb.gimp_image_remove_layer (newImage, ArtisticLayer)
        pdb.gimp_image_remove_layer (newImage, baseforeffect)
        pdb.gimp_image_remove_layer (newImage, newBG)        
         
    #pdb.gimp_display_new (newImage)
      
    stylename =  ["Anguish","Aurora","Barbouillage",
                  "BCGraffiti","Blockism","Bokeh","Brushify",
                  "Cartoon","ChalkItUp","CircleAbstr",
                  "ColorAbstr","ColorPencil","ComicBook",
                  "Cubism","Cutout","DiffTensors",
                  "Doodle","DreamSmoothing",
                  "Ellipsionism",
                  "FeltPen","FingerPaint","Fractalize",
                  "Ghost","GraphicBoost","GraphicNovel",
                  "HardSketch","HighLBloom","HopePoster","HoughSketch",
                  "IllustrationLook","Kuwahara",
                  "LineArt","Linify","LylePaint",
                  "MakeSquiggly","MorphoPaint",
                  "PaintwBrush","Painting",
                  "PastelArt","PenDrawing",
                  "PhotoIll",
                  "PolyDelauney","PolyEnergy",
                  "PosterEdges","Posterize","PosterDither",
                  "QuadtreeVar","Rodilius",
                  "Shapeism","SharpAbstract",
                  "SimpleNoise","Skeletik",
                  "Sketch","SmoothAbstr","Stylize (random style)",
                  "VectorPaint",
                  "Warhol","WhirlDraw",
                 ]
    Framed.name = st+oname+" with a framed artistic effect, style "
    Framed.name = Framed.name+str(stylename[style])
    # optional user choice for preprocessing
    prepsname = ["",    
                "DCP Dehaze (good for light image",
                "Dynamic Range Increase",
                "Equalize Local Histogram",
                "Local Contrast Enhancement",
                "Local Normalization",
                "Local Processing (Equalize)",
                "Local Processing (Normalize)",
                ]
     
    if prepstep>0:
        Framed.name = Framed.name+" (prep.step choosen="+str(prepsname[prepstep])+"."
    #pdb.gimp_display_new (newImage)
    #return
    frameDX = imgW*SH/200
    frameDY = imgH*SV/200
    delta = max(frameDX, frameDY)
    bsize = max(imgW, imgH)
    corner=[0.0,100.0,40.0,30.0,20.0]	
    angle=(float(corner[frame]))
    #pdb.gimp_message ("angle="+str(angle))
     
    if doFlatten==False:
        # create a frame only layer
        # note that the "clear" is approximate, because it seems that
        #   	GMIC and GIMP calculate differently the curves
        #       in the vertices of a round rectangle
        FrameOnly = Framed.copy()
        newImage.add_layer (FrameOnly, 1)
        FrameOnly.name = "Frame Only"
        pdb.gimp_layer_add_alpha (FrameOnly)
        if frame==0:
            pdb.gimp_image_select_rectangle (newImage, CHANNEL_OP_REPLACE,
            delta, delta, imgW, imgH)
        elif frame==1:
            pdb.gimp_image_select_ellipse (newImage, CHANNEL_OP_REPLACE,
            delta, delta, imgW, imgH) 	
        elif frame>1:	
            pdb.gimp_image_select_round_rectangle (newImage, CHANNEL_OP_REPLACE,
            delta, delta, imgW, imgH, angle*imgW/100, angle*imgH/100)  	
        pdb.gimp_drawable_edit_clear (FrameOnly)
        pdb.gimp_selection_none (newImage)
        #pdb.gimp_display_new (newImage)
        #return
        pdb.gimp_item_set_visible (Framed, False)
         		                		
    # align all layers to Framed internal image
    # by MrQ solution
     
    for layer in newImage.layers:		
        pdb.gimp_layer_set_offsets(layer,
            (pdb.gimp_image_width(newImage)-pdb.gimp_drawable_width(layer))/2,
            (pdb.gimp_image_height(newImage)-pdb.gimp_drawable_height(layer))/2)
                         		                		
    pdb.gimp_display_new (newImage)
       
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleFilters/"
copyrightYear="2023"
desc="Artistic and Frame"

register(
    "ArtisticFramed",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Artistic Framed, version 1.1.0 (initial)",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
         "\n (0=same as input)", 3600, (0, 7200, 100)),
        (PF_OPTION, "frame", "Choose frame format", 0,
                     ["square","round","0.2 rounded square",
                      "0.15 rounded square","0.1 rounded square"
                      ]),
        (PF_SPINNER, "size_fr",    "Frame thickness", 20, (0, 50, 2)),					  
        (PF_COLOR,   "color_frame", "Color frame", (200,200,200)),					  
        (PF_OPTION, "effect", "Choose GMIC Artistic effect ", 0,
                     ["Anguish","Aurora","Barbouillage",
                      "BCGraffiti","Blockism","Bokeh","Brushify",
                      "Cartoon","ChalkItUp","CircleAbstr",
                      "ColorAbstr","ColorPencil","ComicBook",
                      "Cubism","Cutout","DiffTensors",
                      "Doodle","DreamSmoothing",
                      "Ellipsionism",
                      "FeltPen","FingerPaint","Fractalize",
                      "Ghost","GraphicBoost","GraphicNovel",
                      "HardSketch","HighLBloom","HopePoster","HoughSketch",
                      "IllustrationLook","Kuwahara",
                      "LineArt","Linify","LylePaint",
                      "MakeSquiggly","MorphoPaint",
                      "PaintwBrush","Painting",
                      "PastelArt","PenDrawing",
                      "PhotoIll",
                      "PolyDelauney","PolyEnergy",
                      "PosterEdges","Posterize","PosterDither",
                      "QuadtreeVar","Rodilius",
                      "Shapeism","SharpAbstract",
                      "SimpleNoise","Skeletik",
                      "Sketch","SmoothAbstr","Stylize",
                      "VectorPaint",
                      "Warhol","WhirlDraw",
                      ]),
        (PF_OPTION, "preproc", "Choose preprocessing step ", 0,
                     ["*none",
                      "DCP Dehaze (good for light images)",
                      "Dynamic Range Increase",
                      "Equalize Local Histogram",
                      "Local Contrast Enhancement",
                      "Local Normalization",
                      "Local Processing (Equalize)",
                      "Local Processing (Normalize)",
                      ]),        
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    ArtisticFramed,
    menu=menu
)

main()
